@@ -1,16 +1,17 @@
 """ Service integration testing best practice.
 """
 
-from nameko.rpc import rpc, rpc_proxy
-from nameko.runners import ServiceRunner
+from nameko.rpc import rpc, RpcProxy
 from nameko.testing.utils import get_container
 from nameko.testing.services import entrypoint_hook
 
 
-class ServiceX(object):
+class ServiceX:
+    """ Service under test
+    """
     name = "service_x"
 
-    y = rpc_proxy("service_y")
+    y = RpcProxy("service_y")
 
     @rpc
     def remote_method(self, value):
@@ -18,27 +19,24 @@ class ServiceX(object):
         return self.y.append_identifier(res)
 
 
-class ServiceY(object):
+class ServiceY:
+    """ Service under test
+    """
     name = "service_y"
 
     @rpc
     def append_identifier(self, value):
         return "{}-y".format(value)
 
-# =============================================================================
-# Begin test
-# =============================================================================
 
-def test_service_integration():
-    config = {'AMQP_URI': 'amqp://guest:guest@localhost:5672/'}
-    runner = ServiceRunner(config)
-    runner.add_service(ServiceX)
-    runner.add_service(ServiceY)
+def test_service_x_y_integration(runner_factory, rabbit_config):
+
+    # run services in the normal manner
+    runner = runner_factory(rabbit_config, ServiceX, ServiceY)
     runner.start()
 
+    # artificially fire the "remote_method" entrypoint on ServiceX
+    # and verify response
     container = get_container(runner, ServiceX)
-
     with entrypoint_hook(container, "remote_method") as entrypoint:
         assert entrypoint("value") == "value-x-y"
-
-    runner.stop()
