@@ -1,54 +1,49 @@
 """
-In a large integration test of highly connected services, it's preferable to
-limit interactions to only those under test.
-
 This file defines several toy services that interact to form a shop of the
 famous ACME Corporation. The AcmeShopService relies on the StockService,
 InvoiceService and PaymentService to fulfil its orders. They are not best
 practice examples! They're minimal services provided for the test at the
 bottom of the file.
 
-``test_shop_integration`` is a  full integration test of the ACME shop
-"checkout" flow. It demonstrates how to test the multiple ACME services in
+``test_shop_integration`` is a full integration test of the ACME shop
+"checkout flow". It demonstrates how to test the multiple ACME services in
 combination with each other, including limiting service interactions by
-replacing certain entrypoint and injection dependencies.
+replacing certain entrypoints and dependencies.
 """
 
 from collections import defaultdict
 
 import pytest
 
-from nameko.dependencies import (
-    InjectionProvider, injection, DependencyFactory)
-from nameko.events import event_dispatcher, Event, event_handler
+from nameko.extensions import DependencyProvider
+from nameko.events import EventDispatcher, event_handler
 from nameko.exceptions import RemoteError
-from nameko.rpc import rpc, rpc_proxy
-from nameko.runners import ServiceRunner
-from nameko.standalone.rpc import RpcProxy
-from nameko.testing.services import replace_injections, restrict_entrypoints
+from nameko.rpc import rpc, RpcProxy
+from nameko.standalone.rpc import ServiceRpcProxy
+from nameko.testing.services import replace_dependencies, restrict_entrypoints
 from nameko.testing.utils import get_container
 from nameko.timer import timer
 
 
-class NotLoggedIn(Exception):
+class NotLoggedInError(Exception):
     pass
 
 
-class ItemOutOfStock(Exception):
+class ItemOutOfStockError(Exception):
     pass
 
 
-class ItemDoesNotExist(Exception):
+class ItemDoesNotExistError(Exception):
     pass
 
 
-class ShoppingBasket(InjectionProvider):
+class ShoppingBasket(DependencyProvider):
     """ A shopping basket tied to the current ``user_id``.
     """
     def __init__(self):
         self.baskets = defaultdict(list)
 
-    def acquire_injection(self, worker_ctx):
+    def get_dependency(self, worker_ctx):
 
         class Basket(object):
             def __init__(self, basket):
@@ -65,37 +60,19 @@ class ShoppingBasket(InjectionProvider):
         try:
             user_id = worker_ctx.data['user_id']
         except KeyError:
-            raise NotLoggedIn()
+            raise NotLoggedInError()
         return Basket(self.baskets[user_id])
 
 
-@injection
-def shopping_basket():
-    """ A shopping basket tied to the current user.
-    """
-    return DependencyFactory(ShoppingBasket)
+class AcmeShopService:
+    name = "acmeshopservice"
 
+    user_basket = ShoppingBasket()
+    stock_rpc = RpcProxy('stockservice')
+    invoice_rpc = RpcProxy('invoiceservice')
+    payment_rpc = RpcProxy('paymentservice')
 
-class ItemAddedToBasket(Event):
-    """ Dispatched when an item is added to a shopping basket.
-    """
-    type = "item_added_to_basket"
-
-
-class CheckoutComplete(Event):
-    """ Dispatched when the checkout process completes
-    """
-    type = "checkout_complete"
-
-
-class AcmeShopService(object):
-
-    user_basket = shopping_basket()
-    stock_service = rpc_proxy('stockservice')
-    invoice_service = rpc_proxy('invoiceservice')
-    payment_service = rpc_proxy('paymentservice')
-
-    fire_event = event_dispatcher()
+    fire_event = EventDispatcher()
 
     @rpc
     def add_to_basket(self, item_code):
@@ -103,37 +80,37 @@ class AcmeShopService(object):
 
         This is a toy example! Ignore the obvious race condition.
         """
-        stock_level = self.stock_service.check_stock(item_code)
+        stock_level = self.stock_rpc.check_stock(item_code)
         if stock_level > 0:
             self.user_basket.add(item_code)
-            self.fire_event(ItemAddedToBasket(item_code))
+            self.fire_event("item_added_to_basket", item_code)
             return item_code
 
-        raise ItemOutOfStock(item_code)
+        raise ItemOutOfStockError(item_code)
 
     @rpc
     def checkout(self):
         """ Take payment for all items in the shopping basket.
         """
-        total_price = sum(self.stock_service.check_price(item)
+        total_price = sum(self.stock_rpc.check_price(item)
                           for item in self.user_basket)
 
         # prepare invoice
-        invoice = self.invoice_service.prepare_invoice(total_price)
+        invoice = self.invoice_rpc.prepare_invoice(total_price)
 
         # take payment
-        self.payment_service.take_payment(invoice)
+        self.payment_rpc.take_payment(invoice)
 
         # fire checkout event if prepare_invoice and take_payment succeeded
-        checkout_event = CheckoutComplete({
+        checkout_event_data = {
             'invoice': invoice,
             'items': list(self.user_basket)
-        })
-        self.fire_event(checkout_event)
+        }
+        self.fire_event("checkout_complete", checkout_event_data)
         return total_price
 
 
-class Warehouse(InjectionProvider):
+class Warehouse(DependencyProvider):
     """ A database of items in the warehouse.
 
     This is a toy example! A dictionary is not a database.
@@ -158,20 +135,14 @@ class Warehouse(InjectionProvider):
             }
         }
 
-    def acquire_injection(self, worker_ctx):
+    def get_dependency(self, worker_ctx):
         return self.database
 
 
-@injection
-def warehouse():
-    """ A shopping basket tied to the current user.
-    """
-    return DependencyFactory(Warehouse)
+class StockService:
+    name = "stockservice"
 
-
-class StockService(object):
-
-    warehouse = warehouse()
+    warehouse = Warehouse()
 
     @rpc
     def check_price(self, item_code):
@@ -180,7 +151,7 @@ class StockService(object):
         try:
             return self.warehouse[item_code]['price']
         except KeyError:
-            raise ItemDoesNotExist(item_code)
+            raise ItemDoesNotExistError(item_code)
 
     @rpc
     def check_stock(self, item_code):
@@ -189,7 +160,7 @@ class StockService(object):
         try:
             return self.warehouse[item_code]['stock']
         except KeyError:
-            raise ItemDoesNotExist(item_code)
+            raise ItemDoesNotExistError(item_code)
 
     @rpc
     @timer(100)
@@ -200,19 +171,19 @@ class StockService(object):
         This is an expensive process that we don't want to exercise during
         integration testing...
         """
-        raise NotImplemented()
+        raise NotImplementedError()
 
-    @event_handler('acmeshopservice', CheckoutComplete.type)
+    @event_handler('acmeshopservice', "checkout_complete")
     def dispatch_items(self, event_data):
         """ Dispatch items from stock on successful checkouts.
 
         This is an expensive process that we don't want to exercise during
         integration testing...
         """
-        raise NotImplemented()
+        raise NotImplementedError()
 
 
-class AddressBook(InjectionProvider):
+class AddressBook(DependencyProvider):
     """ A database of user details, keyed on user_id.
     """
     def __init__(self):
@@ -224,26 +195,20 @@ class AddressBook(InjectionProvider):
             },
         }
 
-    def acquire_injection(self, worker_ctx):
+    def get_dependency(self, worker_ctx):
         def get_user_details():
             try:
                 user_id = worker_ctx.data['user_id']
             except KeyError:
-                raise NotLoggedIn()
+                raise NotLoggedInError()
             return self.address_book.get(user_id)
         return get_user_details
 
 
-@injection
-def address_book():
-    """ Provides the address of the current user.
-    """
-    return DependencyFactory(AddressBook)
+class InvoiceService:
+    name = "invoiceservice"
 
-
-class InvoiceService(object):
-
-    get_user_details = address_book()
+    get_user_details = AddressBook()
 
     @rpc
     def prepare_invoice(self, amount):
@@ -263,7 +228,8 @@ class InvoiceService(object):
         return invoice
 
 
-class PaymentService(object):
+class PaymentService:
+    name = "paymentservice"
 
     @rpc
     def take_payment(self, invoice):
@@ -272,34 +238,13 @@ class PaymentService(object):
         This is an expensive process that we don't want to exercise during
         integration testing...
         """
-        raise NotImplemented()
+        raise NotImplementedError()
 
 # =============================================================================
 # Begin test
 # =============================================================================
 
 
-@pytest.yield_fixture
-def runner_factory(rabbit_config):
-
-    all_runners = []
-
-    def make_runner(*service_classes):
-        runner = ServiceRunner(rabbit_config)
-        for service_cls in service_classes:
-            runner.add_service(service_cls)
-        all_runners.append(runner)
-        return runner
-
-    yield make_runner
-
-    for r in all_runners:
-        try:
-            r.stop()
-        except:
-            pass
-
-
 @pytest.yield_fixture
 def rpc_proxy_factory(rabbit_config):
     """ Factory fixture for standalone RPC proxies.
@@ -311,7 +256,7 @@ def rpc_proxy_factory(rabbit_config):
     all_proxies = []
 
     def make_proxy(service_name, **kwargs):
-        proxy = RpcProxy(service_name, rabbit_config, **kwargs)
+        proxy = ServiceRpcProxy(service_name, rabbit_config, **kwargs)
         all_proxies.append(proxy)
         return proxy.start()
 
@@ -321,14 +266,16 @@ def rpc_proxy_factory(rabbit_config):
         proxy.stop()
 
 
-def test_shop_checkout_integration(runner_factory, rpc_proxy_factory):
+def test_shop_checkout_integration(
+    rabbit_config, runner_factory, rpc_proxy_factory
+):
     """ Simulate a checkout flow as an integration test.
 
     Requires instances of AcmeShopService, StockService and InvoiceService
     to be running. Explicitly replaces the rpc proxy to PaymentService so
     that service doesn't need to be hosted.
 
-    Also replaces the event dispatcher injection on AcmeShopService and
+    Also replaces the event dispatcher dependency on AcmeShopService and
     disables the timer entrypoint on StockService. Limiting the interactions
     of services in this way reduces the scope of the integration test and
     eliminates undesirable side-effects (e.g. processing events unnecessarily).
@@ -336,13 +283,14 @@ def test_shop_checkout_integration(runner_factory, rpc_proxy_factory):
     context_data = {'user_id': 'wile_e_coyote'}
     shop = rpc_proxy_factory('acmeshopservice', context_data=context_data)
 
-    runner = runner_factory(AcmeShopService, StockService, InvoiceService)
+    runner = runner_factory(
+        rabbit_config, AcmeShopService, StockService, InvoiceService)
 
-    # replace ``event_dispatcher`` and ``payment_service``  injections on
-    # AcmeShopService with Mock injections
+    # replace ``event_dispatcher`` and ``payment_rpc``  dependencies on
+    # AcmeShopService with ``MockDependencyProvider``\s
     shop_container = get_container(runner, AcmeShopService)
-    fire_event, payment_service = replace_injections(
-        shop_container, "fire_event", "payment_service")
+    fire_event, payment_rpc = replace_dependencies(
+        shop_container, "fire_event", "payment_rpc")
 
     # restrict entrypoints on StockService
     stock_container = get_container(runner, StockService)
@@ -357,10 +305,10 @@ def test_shop_checkout_integration(runner_factory, rpc_proxy_factory):
     # try to buy something that's out of stock
     with pytest.raises(RemoteError) as exc_info:
         shop.add_to_basket("toothpicks")
-    assert exc_info.value.exc_type == "ItemOutOfStock"
+    assert exc_info.value.exc_type == "ItemOutOfStockError"
 
     # provide a mock response from the payment service
-    payment_service.take_payment.return_value = "Payment complete."
+    payment_rpc.take_payment.return_value = "Payment complete."
 
     # checkout
     res = shop.checkout()
@@ -369,7 +317,7 @@ def test_shop_checkout_integration(runner_factory, rpc_proxy_factory):
     assert res == total_amount
 
     # verify integration with mocked out payment service
-    payment_service.take_payment.assert_called_once_with({
+    payment_rpc.take_payment.assert_called_once_with({
         'customer': "wile_e_coyote",
         'address': "12 Long Road, High Cliffs, Utah",
         'amount': total_amount,
