@@ -1,7 +1,18 @@
+from __future__ import unicode_literals
+
 import inspect
+import sys
+
+import six
 
 
-class DependencyNotFound(AttributeError):
+if sys.version_info >= (3, 3):  # pragma: no cover
+    from collections.abc import Iterable  # pylint: disable=E0611,E0401
+else:  # pragma: no cover
+    from collections import Iterable
+
+
+class ExtensionNotFound(AttributeError):
     pass
 
 
@@ -9,17 +20,16 @@ class RpcTimeout(Exception):
     pass
 
 
-class RpcConnectionError(Exception):
-    """Raised (in the caller) if the connection to the broker is lost while
-    waiting for an rpc reply"""
+class ReplyQueueExpiredWithPendingReplies(Exception):
+    pass
 
 
 class ContainerBeingKilled(Exception):
     """Raised by :meth:`Container.spawn_worker` if it has started a ``kill``
     sequence.
 
-    Entrypoint providers should catch this and react as if they hadn't been
-    available in the first place, e.g. an rpc consumer should probably requeue
+    Entrypoints should catch this and react as if they hadn't been available
+    in the first place, e.g. an rpc consumer should probably requeue
     the message.
 
     We need this because eventlet may yield during the execution of
@@ -45,7 +55,7 @@ def get_module_path(exc_type):
 
 
 class RemoteError(Exception):
-    """ Exception to raise at the caller if an exception occured in the
+    """ Exception to raise at the caller if an exception occurred in the
     remote worker.
     """
     def __init__(self, exc_type=None, value=""):
@@ -55,19 +65,38 @@ class RemoteError(Exception):
         super(RemoteError, self).__init__(message)
 
 
+def safe_for_serialization(value):
+    """ Transform a value in preparation for serializing as json
+
+    no-op for strings, mappings and iterables have their entries made safe,
+    and all other values are stringified, with a fallback value if that fails
+    """
+
+    if isinstance(value, six.string_types):
+        return value
+    if isinstance(value, dict):
+        return {
+            safe_for_serialization(key): safe_for_serialization(val)
+            for key, val in six.iteritems(value)
+        }
+    if isinstance(value, Iterable):
+        return list(map(safe_for_serialization, value))
+
+    try:
+        return six.text_type(value)
+    except Exception:
+        return '[__unicode__ failed]'
+
+
 def serialize(exc):
     """ Serialize `self.exc` into a data dictionary representing it.
     """
-    try:
-        value = unicode(exc)
-    except Exception:
-        value = '[__unicode__ failed]'
 
     return {
         'exc_type': type(exc).__name__,
         'exc_path': get_module_path(type(exc)),
-        'exc_args': exc.args,
-        'value': value,
+        'exc_args': list(map(safe_for_serialization, exc.args)),
+        'value': safe_for_serialization(exc),
     }
 
 
@@ -77,7 +106,7 @@ def deserialize(data):
     If the `exc_path` value matches an exception registered as
     ``deserializable``, return an instance of that exception type.
     Otherwise, return a `RemoteError` instance describing the exception
-    that occured.
+    that occurred.
     """
     key = data.get('exc_path')
     if key in registry:
@@ -98,18 +127,22 @@ def deserialize_to_instance(exc_type):
     return exc_type
 
 
-@deserialize_to_instance
-class MalformedRequest(Exception):
+class BadRequest(Exception):
     pass
 
 
 @deserialize_to_instance
-class MethodNotFound(Exception):
+class MalformedRequest(BadRequest):
     pass
 
 
 @deserialize_to_instance
-class IncorrectSignature(Exception):
+class MethodNotFound(BadRequest):
+    pass
+
+
+@deserialize_to_instance
+class IncorrectSignature(BadRequest):
     pass
 
 
@@ -132,3 +165,15 @@ class UnserializableValueError(Exception):
 
     def __str__(self):
         return "Unserializable value: `{}`".format(self.repr_value)
+
+
+class ConfigurationError(Exception):
+    pass
+
+
+class CommandError(Exception):
+    """Raise from subcommands to report error back to the user"""
+
+
+class ConnectionNotFound(BadRequest):
+    """Unknown websocket connection id"""
