@@ -1,27 +1,33 @@
+# coding: utf-8
+
 import sys
+from functools import partial
 
 import eventlet
-from eventlet import spawn, sleep, Timeout
-from eventlet.event import Event
 import greenlet
-from mock import patch, call, ANY, Mock
 import pytest
+from eventlet import Timeout, sleep, spawn
+from eventlet.event import Event
+from mock import ANY, Mock, call, patch
 
-from nameko.containers import ServiceContainer, WorkerContext
 from nameko.constants import MAX_WORKERS_CONFIG_KEY
-from nameko.dependencies import(
-    InjectionProvider, EntrypointProvider, entrypoint, injection,
-    DependencyFactory)
-from nameko.testing.utils import AnyInstanceOf, get_dependency
+from nameko.containers import ServiceContainer, get_service_name
+from nameko.exceptions import ConfigurationError
+from nameko.extensions import DependencyProvider, Entrypoint
+from nameko.testing.utils import get_extension
 
 
 class CallCollectorMixin(object):
     call_counter = 0
 
     def __init__(self):
-        self.instances.add(self)
         self._reset_calls()
 
+    def bind(self, *args):
+        res = super(CallCollectorMixin, self).bind(*args)
+        self.instances.add(res)
+        return res
+
     def _reset_calls(self):
         self.calls = []
         self.call_ids = []
@@ -31,9 +37,9 @@ class CallCollectorMixin(object):
         self.calls.append(data)
         self.call_ids.append(CallCollectorMixin.call_counter)
 
-    def prepare(self):
-        self._log_call(('prepare'))
-        super(CallCollectorMixin, self).prepare()
+    def setup(self):
+        self._log_call(('setup'))
+        super(CallCollectorMixin, self).setup()
 
     def start(self):
         self._log_call(('start'))
@@ -47,43 +53,34 @@ class CallCollectorMixin(object):
         self._log_call(('kill'))
         super(CallCollectorMixin, self).stop()
 
+
+class CallCollectingEntrypoint(CallCollectorMixin, Entrypoint):
+    instances = set()
+    expected_exceptions = ()
+
+
+class CallCollectingDependencyProvider(CallCollectorMixin, DependencyProvider):
+    instances = set()
+
+    def get_dependency(self, worker_ctx):
+        self._log_call(('get_dependency', worker_ctx))
+        return 'spam-attr'
+
     def worker_setup(self, worker_ctx):
-        self._log_call(('setup', worker_ctx))
+        self._log_call(('worker_setup', worker_ctx))
         super(CallCollectorMixin, self).worker_setup(worker_ctx)
 
     def worker_result(self, worker_ctx, result=None, exc_info=None):
-        self._log_call(('result', worker_ctx, (result, exc_info)))
+        self._log_call(('worker_result', worker_ctx, (result, exc_info)))
         super(CallCollectorMixin, self).worker_result(
             worker_ctx, result, exc_info)
 
     def worker_teardown(self, worker_ctx):
-        self._log_call(('teardown', worker_ctx))
+        self._log_call(('worker_teardown', worker_ctx))
         super(CallCollectorMixin, self).worker_teardown(worker_ctx)
 
 
-class CallCollectingEntrypointProvider(
-        CallCollectorMixin, EntrypointProvider):
-    instances = set()
-
-
-class CallCollectingInjectionProvider(
-        CallCollectorMixin, InjectionProvider):
-    instances = set()
-
-    def acquire_injection(self, worker_ctx):
-        self._log_call(('acquire', worker_ctx))
-        return 'spam-attr'
-
-
-@entrypoint
-def foobar():
-    return DependencyFactory(CallCollectingEntrypointProvider)
-
-
-@injection
-def call_collector():
-    return DependencyFactory(CallCollectingInjectionProvider)
-
+foobar = CallCollectingEntrypoint.decorator
 
 egg_error = Exception('broken')
 
@@ -91,7 +88,7 @@ egg_error = Exception('broken')
 class Service(object):
     name = 'test-service'
 
-    spam = call_collector()
+    spam = CallCollectingDependencyProvider()
 
     @foobar
     def ham(self):
@@ -101,19 +98,12 @@ class Service(object):
     def egg(self):
         raise egg_error
 
-    @foobar
-    def wait(self):
-        while True:
-            sleep()
-
 
 @pytest.fixture
 def container():
-    container = ServiceContainer(service_cls=Service,
-                                 worker_ctx_cls=WorkerContext,
-                                 config={})
-    for dep in container.dependencies:
-        dep._reset_calls()
+    container = ServiceContainer(Service, config={})
+    for ext in container.extensions:
+        ext._reset_calls()
 
     CallCollectorMixin.call_counter = 0
     return container
@@ -125,52 +115,50 @@ def logger():
         yield patched
 
 
-def test_collects_dependencies(container):
-    assert len(container.dependencies) == 4
-    assert container.dependencies == (
-        CallCollectingEntrypointProvider.instances |
-        CallCollectingInjectionProvider.instances)
+def test_collects_extensions(container):
+    assert len(container.extensions) == 3
+    assert container.extensions == (
+        CallCollectingEntrypoint.instances |
+        CallCollectingDependencyProvider.instances)
 
 
-def test_starts_dependencies(container):
+def test_starts_extensions(container):
 
-    for dep in container.dependencies:
-        assert dep.calls == []
+    for ext in container.extensions:
+        assert ext.calls == []
 
     container.start()
 
-    for dep in container.dependencies:
-        assert dep.calls == [
-            ('prepare'),
+    for ext in container.extensions:
+        assert ext.calls == [
+            ('setup'),
             ('start')
         ]
 
 
-def test_stops_dependencies(container):
+def test_stops_extensions(container):
 
     container.stop()
-    for dep in container.dependencies:
-        assert dep.calls == [
+    for ext in container.extensions:
+        assert ext.calls == [
             ('stop')
         ]
 
 
-def test_stops_entrypoints_before_injections(container):
+def test_stops_entrypoints_before_dependency_providers(container):
     container.stop()
 
-    dependencies = container.dependencies
-    spam_dep = get_dependency(container, InjectionProvider)
+    provider = get_extension(container, DependencyProvider)
 
-    for dec_dep in dependencies.entrypoints:
-        assert dec_dep.call_ids[0] < spam_dep.call_ids[0]
+    for entrypoint in container.entrypoints:
+        assert entrypoint.call_ids[0] < provider.call_ids[0]
 
 
 def test_worker_life_cycle(container):
-    dependencies = container.dependencies
 
-    (spam_dep,) = [dep for dep in dependencies if dep.name == 'spam']
-    (ham_dep,) = [dep for dep in dependencies if dep.name == 'ham']
-    (egg_dep,) = [dep for dep in dependencies if dep.name == 'egg']
+    spam_dep = get_extension(container, DependencyProvider)
+    ham_dep = get_extension(container, Entrypoint, method_name="ham")
+    egg_dep = get_extension(container, Entrypoint, method_name="egg")
 
     handle_result = Mock()
     handle_result.side_effect = (
@@ -185,28 +173,14 @@ def test_worker_life_cycle(container):
     container._worker_pool.waitall()
 
     assert spam_dep.calls == [
-        ('acquire', ham_worker_ctx),
-        ('setup', ham_worker_ctx),
-        ('result', ham_worker_ctx, ('ham', None)),
-        ('teardown', ham_worker_ctx),
-        ('acquire', egg_worker_ctx),
-        ('setup', egg_worker_ctx),
-        ('result', egg_worker_ctx, (None, (Exception, egg_error, ANY))),
-        ('teardown', egg_worker_ctx),
-    ]
-
-    assert ham_dep.calls == [
-        ('setup', ham_worker_ctx),
-        ('teardown', ham_worker_ctx),
-        ('setup', egg_worker_ctx),
-        ('teardown', egg_worker_ctx),
-    ]
-
-    assert egg_dep.calls == [
-        ('setup', ham_worker_ctx),
-        ('teardown', ham_worker_ctx),
-        ('setup', egg_worker_ctx),
-        ('teardown', egg_worker_ctx),
+        ('get_dependency', ham_worker_ctx),
+        ('worker_setup', ham_worker_ctx),
+        ('worker_result', ham_worker_ctx, ('ham', None)),
+        ('worker_teardown', ham_worker_ctx),
+        ('get_dependency', egg_worker_ctx),
+        ('worker_setup', egg_worker_ctx),
+        ('worker_result', egg_worker_ctx, (None, (Exception, egg_error, ANY))),
+        ('worker_teardown', egg_worker_ctx),
     ]
 
     assert handle_result.call_args_list == [
@@ -237,16 +211,14 @@ def test_container_doesnt_exhaust_max_workers(container):
             spam_called.send(a)
             spam_continue.wait()
 
-    container = ServiceContainer(service_cls=Service,
-                                 worker_ctx_cls=WorkerContext,
-                                 config={MAX_WORKERS_CONFIG_KEY: 1})
+    container = ServiceContainer(Service, config={MAX_WORKERS_CONFIG_KEY: 1})
 
-    dep = get_dependency(container, EntrypointProvider)
+    dep = get_extension(container, Entrypoint)
 
     # start the first worker, which should wait for spam_continue
     container.spawn_worker(dep, ['ham'], {})
 
-    # start the next worker in a speparate thread,
+    # start the next worker in a separate thread,
     # because it should block until the first one completed
     gt = spawn(container.spawn_worker, dep, ['eggs'], {})
 
@@ -284,7 +256,7 @@ def test_kill_already_stopped(container, logger):
     assert logger.debug.call_args == call("already stopped %s", container)
 
 
-def test_kill_container_with_active_threads(container):
+def test_kill_container_with_managed_threads(container):
     """ Start a thread that's not managed by dependencies. Ensure it is killed
     when the container is.
     """
@@ -294,30 +266,8 @@ def test_kill_container_with_active_threads(container):
 
     container.spawn_managed_thread(sleep_forever)
 
-    assert len(container._active_threads) == 1
-    (worker_gt,) = container._active_threads
-
-    container.kill()
-
-    with Timeout(1):
-        container._died.wait()
-
-        with pytest.raises(greenlet.GreenletExit):
-            worker_gt.wait()
-
-
-def test_kill_container_with_protected_threads(container):
-    """ Start a protected thread that's not managed by dependencies. Ensure it
-    is killed when the container is.
-    """
-    def sleep_forever():
-        while True:
-            sleep()
-
-    container.spawn_managed_thread(sleep_forever, protected=True)
-
-    assert len(container._protected_threads) == 1
-    (worker_gt,) = container._protected_threads
+    assert len(container._managed_threads) == 1
+    (worker_gt,) = container._managed_threads.keys()
 
     container.kill()
 
@@ -341,7 +291,7 @@ def test_kill_container_with_active_workers(container_factory):
             wait_forever.wait()
 
     container = container_factory(Service, {})
-    dep = get_dependency(container, EntrypointProvider)
+    dep = get_extension(container, Entrypoint)
 
     # start the first worker, which should wait for spam_continue
     container.spawn_worker(dep, (), {})
@@ -350,23 +300,24 @@ def test_kill_container_with_active_workers(container_factory):
 
     with patch('nameko.containers._log') as logger:
         container.kill()
-    calls = logger.warning.call_args_list
-    assert call(
-        'killing active thread for %s', 'kill-with-active-workers.spam'
-    ) in calls
+
+    assert logger.warning.call_args_list == [
+        call('killing %s active workers(s)', 1),
+        call('killing active worker for %s', ANY)
+    ]
 
 
 def test_handle_killed_worker(container, logger):
 
-    dep = get_dependency(container, EntrypointProvider)
+    dep = get_extension(container, Entrypoint)
     container.spawn_worker(dep, ['sleep'], {})
 
-    assert len(container._active_threads) == 1
-    (worker_gt,) = container._active_threads
+    assert len(container._worker_threads) == 1
+    (worker_gt,) = container._worker_threads.values()
 
     worker_gt.kill()
-    assert logger.warning.call_args == call("%s thread killed by container",
-                                            container)
+    assert logger.debug.call_args == call(
+        "%s thread killed by container", container)
 
     assert not container._died.ready()  # container continues running
 
@@ -417,12 +368,10 @@ def test_container_only_killed_once(container):
         raise exc
 
     with patch.object(
-            container, '_kill_active_threads', autospec=True) as kill_threads:
+        container, '_kill_managed_threads', autospec=True
+    ) as kill_managed_threads:
 
         with patch.object(container, 'kill', wraps=container.kill) as kill:
-            # insert an eventlet yield into the kill process, otherwise
-            # the container dies before the second exception gets raised
-            kill.side_effect = lambda exc: sleep()
 
             container.start()
 
@@ -444,7 +393,7 @@ def test_container_only_killed_once(container):
             ]
 
             # only the first kill results in any cleanup
-            assert kill_threads.call_count == 1
+            assert kill_managed_threads.call_count == 1
 
 
 def test_container_stop_kills_remaining_managed_threads(container, logger):
@@ -458,14 +407,20 @@ def test_container_stop_kills_remaining_managed_threads(container, logger):
     container.start()
 
     container.spawn_managed_thread(sleep_forever)
-    container.spawn_managed_thread(sleep_forever, protected=True)
+    container.spawn_managed_thread(sleep_forever)
 
     container.stop()
 
     assert logger.warning.call_args_list == [
-        call("killing %s active thread(s)", 1),
+        call("killing %s managed thread(s)", 2),
+        call("killing managed thread `%s`", "sleep_forever"),
+        call("killing managed thread `%s`", "sleep_forever"),
+    ]
+
+    assert logger.debug.call_args_list == [
+        call("starting %s", container),
+        call("stopping %s", container),
         call("%s thread killed by container", container),
-        call("killing %s protected thread(s)", 1),
         call("%s thread killed by container", container),
     ]
 
@@ -481,14 +436,23 @@ def test_container_kill_kills_remaining_managed_threads(container, logger):
     container.start()
 
     container.spawn_managed_thread(sleep_forever)
-    container.spawn_managed_thread(sleep_forever, protected=True)
+    container.spawn_managed_thread(sleep_forever)
 
     container.kill()
 
     assert logger.warning.call_args_list == [
-        call("killing %s active thread(s)", 1),
+        call("killing %s managed thread(s)", 2),
+        call("killing managed thread `%s`", "sleep_forever"),
+        call("killing managed thread `%s`", "sleep_forever"),
+    ]
+
+    assert logger.info.call_args_list == [
+        call("killing %s", container),
+    ]
+
+    assert logger.debug.call_args_list == [
+        call("starting %s", container),
         call("%s thread killed by container", container),
-        call("killing %s protected thread(s)", 1),
         call("%s thread killed by container", container),
     ]
 
@@ -497,7 +461,7 @@ def test_kill_bad_dependency(container):
     """ Verify that an exception from a badly-behaved dependency.kill()
     doesn't stop the container's kill process.
     """
-    dep = get_dependency(container, InjectionProvider)
+    dep = get_extension(container, DependencyProvider)
     with patch.object(dep, 'kill') as dep_kill:
         dep_kill.side_effect = Exception('dependency error')
 
@@ -507,14 +471,13 @@ def test_kill_bad_dependency(container):
         try:
             raise Exception('container error')
         except:
-            pass
-        exc_info = sys.exc_info()
+            exc_info = sys.exc_info()
 
         container.kill(exc_info)
 
         with pytest.raises(Exception) as exc_info:
             container.wait()
-        assert exc_info.value.message == "container error"
+        assert str(exc_info.value) == "container error"
 
 
 def test_stop_during_kill(container, logger):
@@ -522,17 +485,17 @@ def test_stop_during_kill(container, logger):
     a container while it is being killed.
     """
     with patch.object(
-            container, '_kill_active_threads', autospec=True) as kill_threads:
+        container, '_kill_managed_threads', autospec=True
+    ) as kill_managed_threads:
 
         # force eventlet yield during kill() so stop() will be scheduled
-        kill_threads.side_effect = eventlet.sleep
+        kill_managed_threads.side_effect = eventlet.sleep
 
         # manufacture an exc_info to kill with
         try:
             raise Exception('error')
         except:
-            pass
-        exc_info = sys.exc_info()
+            exc_info = sys.exc_info()
 
         eventlet.spawn(container.kill, exc_info)
         eventlet.spawn(container.stop)
@@ -544,13 +507,52 @@ def test_stop_during_kill(container, logger):
         ]
 
 
-def test_container_entrypoint_property(container):
-    entrypoints = container.entrypoints
-    assert {dep.name for dep in entrypoints} == {'wait', 'ham', 'egg'}
-    assert entrypoints == 3 * [AnyInstanceOf(CallCollectingEntrypointProvider)]
+def test_get_service_name():
+
+    class Service():
+        name = "str"
+
+    class UnicodeService():
+        name = u"unicøde"
+
+    class BadNameService():
+        name = object()
+
+    class AnonymousService():
+        pass
+
+    assert get_service_name(Service) == "str"
+    assert get_service_name(UnicodeService) == u"unicøde"
+
+    with pytest.raises(ConfigurationError) as exc_info:
+        get_service_name(BadNameService)
+    assert str(exc_info.value) == (
+        'Service name attribute must be a string '
+        '(test.test_container.BadNameService.name)'
+    )
+
+    with pytest.raises(ConfigurationError) as exc_info:
+        get_service_name(AnonymousService)
+    assert str(exc_info.value) == (
+        'Service class must define a `name` attribute '
+        '(test.test_container.AnonymousService)'
+    )
 
 
-def test_container_injection_property(container):
-    injections = container.injections
-    assert {dep.name for dep in injections} == {'spam'}
-    assert injections == [AnyInstanceOf(CallCollectingInjectionProvider)]
+def test_logging_managed_threads(container, logger):
+
+    def wait():
+        Event().wait()
+
+    container.spawn_managed_thread(wait)  # anonymous
+    container.spawn_managed_thread(partial(wait))  # __name__less
+    container.spawn_managed_thread(wait, identifier='named')
+
+    container.stop()
+
+    # order is not guaranteed when killing threads
+    call_args_list = logger.warning.call_args_list
+    assert call("killing %s managed thread(s)", 3) in call_args_list
+    assert call("killing managed thread `%s`", "wait") in call_args_list
+    assert call("killing managed thread `%s`", "<unknown>") in call_args_list
+    assert call("killing managed thread `%s`", "named") in call_args_list
