@@ -1,16 +1,14 @@
 # start a runner with a service that errors. does it hang? or stop?
 # how do we get an individual servicecontainer to blow up?
-import eventlet
-from mock import patch
 import pytest
+from mock import ANY, call, patch
 
-
+from nameko.events import EventDispatcher
 from nameko.exceptions import RemoteError
-from nameko.events import event_dispatcher, EventDispatcher
-from nameko.rpc import rpc, rpc_proxy, RpcConsumer
-from nameko.standalone.rpc import RpcProxy
-from nameko.testing.utils import get_dependency, get_container
+from nameko.rpc import RpcConsumer, RpcProxy, rpc
+from nameko.standalone.rpc import ServiceRpcProxy
 from nameko.testing.services import entrypoint_hook
+from nameko.testing.utils import get_container, get_extension
 
 
 class ExampleError(Exception):
@@ -18,9 +16,10 @@ class ExampleError(Exception):
 
 
 class ExampleService(object):
+    name = "exampleservice"
 
-    dispatch = event_dispatcher()
-    rpcproxy = rpc_proxy('exampleservice')
+    dispatch = EventDispatcher()
+    rpcproxy = RpcProxy('exampleservice')
 
     @rpc
     def task(self):
@@ -30,6 +29,10 @@ class ExampleService(object):
     def broken(self):
         raise ExampleError("broken")
 
+    @rpc(expected_exceptions=ExampleError)
+    def bad(self):
+        raise ExampleError("bad")
+
     @rpc
     def proxy(self, method, *args):
         """ Proxies RPC calls to ``method`` on itself, so we can test handling
@@ -39,9 +42,11 @@ class ExampleService(object):
 
 
 class SecondService(object):
+    name = "secondservice"
+
     @rpc
     def task(self):
-        return 'task_result'
+        return 'task_result'  # pragma: no cover
 
 
 def test_error_in_worker(container_factory, rabbit_config):
@@ -49,11 +54,30 @@ def test_error_in_worker(container_factory, rabbit_config):
     container = container_factory(ExampleService, rabbit_config)
     container.start()
 
-    with entrypoint_hook(container, "broken") as broken:
-        with pytest.raises(ExampleError):
-            broken()
+    with patch('nameko.containers._log') as logger:
+        with entrypoint_hook(container, "broken") as broken:
+            with pytest.raises(ExampleError):
+                broken()
     assert not container._died.ready()
 
+    assert logger.exception.call_args == call(
+        'error handling worker %s: %s', ANY, ANY)
+
+
+def test_expected_error_in_worker(container_factory, rabbit_config):
+
+    container = container_factory(ExampleService, rabbit_config)
+    container.start()
+
+    with patch('nameko.containers._log') as logger:
+        with entrypoint_hook(container, "bad") as bad:
+            with pytest.raises(ExampleError):
+                bad()
+    assert not container._died.ready()
+
+    assert logger.warning.call_args == call(
+        '(expected) error handling worker %s: %s', ANY, ANY, exc_info=True)
+
 
 def test_error_in_remote_worker(container_factory, rabbit_config):
 
@@ -72,25 +96,22 @@ def test_handle_result_error(container_factory, rabbit_config):
     container = container_factory(ExampleService, rabbit_config)
     container.start()
 
-    rpc_consumer = get_dependency(container, RpcConsumer)
+    rpc_consumer = get_extension(container, RpcConsumer)
     with patch.object(
-            rpc_consumer, 'handle_result', autospec=True) as handle_result:
+            rpc_consumer, 'handle_result', autospec=True
+    ) as handle_result:
         err = "error in handle_result"
         handle_result.side_effect = Exception(err)
 
         # use a standalone rpc proxy to call exampleservice.task()
-        with RpcProxy("exampleservice", rabbit_config) as proxy:
-            # proxy.task() will never return, so give up almost immediately
-            try:
-                with eventlet.Timeout(0):
-                    proxy.task()
-            except eventlet.Timeout:
-                pass
+        with ServiceRpcProxy("exampleservice", rabbit_config) as proxy:
+            # proxy.task() will hang forever because it generates an error
+            proxy.task.call_async()
 
-        with eventlet.Timeout(1):
-            with pytest.raises(Exception) as exc_info:
-                container.wait()
-            assert exc_info.value.message == err
+        with pytest.raises(Exception) as exc_info:
+            container.wait()
+
+        assert str(exc_info.value) == err
 
 
 @pytest.mark.parametrize("method_name",
@@ -101,27 +122,20 @@ def test_dependency_call_lifecycle_errors(
     container = container_factory(ExampleService, rabbit_config)
     container.start()
 
-    dependency = get_dependency(container, EventDispatcher)
+    dependency = get_extension(container, EventDispatcher)
     with patch.object(dependency, method_name, autospec=True) as method:
         err = "error in {}".format(method_name)
         method.side_effect = Exception(err)
 
         # use a standalone rpc proxy to call exampleservice.task()
-        with RpcProxy("exampleservice", rabbit_config) as proxy:
+        with ServiceRpcProxy("exampleservice", rabbit_config) as proxy:
             # proxy.task() will hang forever because it generates an error
-            # in the remote container (so never receives a response).
-            # generate and then swallow a timeout as soon as the thread yields
-            try:
-                with eventlet.Timeout(0):
-                    proxy.task()
-            except eventlet.Timeout:
-                pass
+            proxy.task.call_async()
 
         # verify that the error bubbles up to container.wait()
-        with eventlet.Timeout(1):
-            with pytest.raises(Exception) as exc_info:
-                container.wait()
-            assert exc_info.value.message == err
+        with pytest.raises(Exception) as exc_info:
+            container.wait()
+        assert str(exc_info.value) == err
 
 
 def test_runner_catches_container_errors(runner_factory, rabbit_config):
@@ -131,22 +145,17 @@ def test_runner_catches_container_errors(runner_factory, rabbit_config):
 
     container = get_container(runner, ExampleService)
 
-    rpc_consumer = get_dependency(container, RpcConsumer)
+    rpc_consumer = get_extension(container, RpcConsumer)
     with patch.object(
             rpc_consumer, 'handle_result', autospec=True) as handle_result:
         exception = Exception("error")
         handle_result.side_effect = exception
 
         # use a standalone rpc proxy to call exampleservice.task()
-        with RpcProxy("exampleservice", rabbit_config) as proxy:
+        with ServiceRpcProxy("exampleservice", rabbit_config) as proxy:
             # proxy.task() will hang forever because it generates an error
             # in the remote container (so never receives a response).
-            # generate and then swallow a timeout as soon as the thread yields
-            try:
-                with eventlet.Timeout(0):
-                    proxy.task()
-            except eventlet.Timeout:
-                pass
+            proxy.task.call_async()
 
         # verify that the error bubbles up to runner.wait()
         with pytest.raises(Exception) as exc_info:
@@ -164,23 +173,17 @@ def test_graceful_stop_on_one_container_error(runner_factory, rabbit_config):
     original_stop = second_container.stop
     with patch.object(second_container, 'stop', autospec=True,
                       wraps=original_stop) as stop:
-        rpc_consumer = get_dependency(container, RpcConsumer)
+        rpc_consumer = get_extension(container, RpcConsumer)
         with patch.object(
                 rpc_consumer, 'handle_result', autospec=True) as handle_result:
             exception = Exception("error")
             handle_result.side_effect = exception
 
             # use a standalone rpc proxy to call exampleservice.task()
-            with RpcProxy("exampleservice", rabbit_config) as proxy:
+            with ServiceRpcProxy("exampleservice", rabbit_config) as proxy:
                 # proxy.task() will hang forever because it generates an error
                 # in the remote container (so never receives a response).
-                # generate and then swallow a timeout as soon as the thread
-                # yields
-                try:
-                    with eventlet.Timeout(0):
-                        proxy.task()
-                except eventlet.Timeout:
-                    pass
+                proxy.task.call_async()
 
             # verify that the error bubbles up to runner.wait()
             with pytest.raises(Exception) as exc_info:
