@@ -1,39 +1,34 @@
 import json
 import sys
 
-from mock import ANY
 import pytest
+from mock import ANY
 
-from nameko.dependencies import (
-    InjectionProvider, DependencyFactory, injection, entrypoint)
 from nameko.exceptions import RemoteError
-from nameko.rpc import RpcProvider
-from nameko.standalone.rpc import RpcProxy
-from nameko.testing.utils import wait_for_worker_idle
+from nameko.extensions import DependencyProvider
+from nameko.rpc import Rpc
+from nameko.standalone.rpc import ServiceRpcProxy
+from nameko.testing.services import entrypoint_waiter
 
 
 worker_result_called = []
 
 
-@pytest.fixture(autouse=True)
+@pytest.yield_fixture(autouse=True)
 def reset():
+    yield
     del worker_result_called[:]
 
 
-class CollectorInjection(InjectionProvider):
-    """ InjectionProvider that collects worker results
+class ResultCollector(DependencyProvider):
+    """ DependencyProvider that collects worker results
     """
     def worker_result(self, worker_ctx, res, exc_info):
         worker_result_called.append((res, exc_info))
 
 
-@injection
-def result_collector():
-    return DependencyFactory(CollectorInjection)
-
-
-class CustomRpcProvider(RpcProvider):
-    """ RpcProvider subclass that verifies `result` can be serialized to json,
+class CustomRpc(Rpc):
+    """ Rpc subclass that verifies `result` can be serialized to json,
     and changes the `result` and `exc_info` accordingly.
     """
     def handle_result(self, message, worker_ctx, result, exc_info):
@@ -43,18 +38,17 @@ class CustomRpcProvider(RpcProvider):
             result = "something went wrong"
             exc_info = sys.exc_info()
 
-        return super(CustomRpcProvider, self).handle_result(
+        return super(CustomRpc, self).handle_result(
             message, worker_ctx, result, exc_info)
 
 
-@entrypoint
-def custom_rpc():
-    return DependencyFactory(CustomRpcProvider)
+custom_rpc = CustomRpc.decorator
 
 
 class ExampleService(object):
+    name = "exampleservice"
 
-    collector = result_collector()
+    collector = ResultCollector()
 
     @custom_rpc
     def echo(self, arg):
@@ -65,23 +59,31 @@ class ExampleService(object):
         return object()
 
 
-def test_handle_result(container_factory, rabbit_manager, rabbit_config):
+@pytest.yield_fixture
+def rpc_proxy(rabbit_config):
+    with ServiceRpcProxy('exampleservice', rabbit_config) as proxy:
+        yield proxy
+
+
+def test_handle_result(
+    container_factory, rabbit_manager, rabbit_config, rpc_proxy
+):
     """ Verify that `handle_result` can modify the return values of the worker,
     such that other dependencies see the updated values.
     """
     container = container_factory(ExampleService, rabbit_config)
     container.start()
 
-    with RpcProxy('exampleservice', rabbit_config) as proxy:
+    assert rpc_proxy.echo("hello") == "hello"
 
-        assert proxy.echo("hello") == "hello"
+    # use entrypoint_waiter because the rpc proxy returns before
+    # worker_result is called
+    with entrypoint_waiter(container, "unserializable"):
         with pytest.raises(RemoteError) as exc:
-            proxy.unserializable()
-        assert "is not JSON serializable" in exc.value.message
+            rpc_proxy.unserializable()
+        assert "is not JSON serializable" in str(exc.value)
 
-    wait_for_worker_idle(container)
-
-    # verify CollectorInjection sees values returned from `handle_result`
+    # verify ResultCollector sees values returned from `handle_result`
     assert worker_result_called == [
         ("hello", None),
         ("something went wrong", (TypeError, ANY, ANY)),
