@@ -1,11 +1,12 @@
-import eventlet
 import pytest
+from mock import Mock, call
 
-from nameko.events import event_handler, Event, BROADCAST
-from nameko.standalone.events import event_dispatcher
-from nameko.standalone.rpc import RpcProxy
+from nameko.events import BROADCAST, event_handler
 from nameko.rpc import rpc
 from nameko.runners import ServiceRunner, run_services
+from nameko.standalone.events import event_dispatcher
+from nameko.standalone.rpc import ServiceRpcProxy
+from nameko.testing.services import entrypoint_waiter
 from nameko.testing.utils import assert_stops_raising, get_container
 
 
@@ -17,35 +18,35 @@ class TestService2(object):
     name = 'foobar_2'
 
 
-received = []
+@pytest.fixture
+def tracker():
+    return Mock()
 
 
-@pytest.fixture(autouse=True)
-def reset_mock():
-    del received[:]
+@pytest.fixture
+def service_cls(tracker):
+
+    class Service(object):
+        name = "service"
+
+        @rpc
+        @event_handler(
+            "srcservice", "testevent",
+            handler_type=BROADCAST, reliable_delivery=False
+        )
+        def handle(self, msg):
+            tracker(msg)
+
+    return Service
 
 
-class TestEvent(Event):
-    type = "testevent"
-
-
-class Service(object):
-
-    @rpc
-    @event_handler("srcservice", TestEvent.type, handler_type=BROADCAST,
-                   reliable_delivery=False)
-    def handle(self, msg):
-        received.append(msg)
-
-
-def test_runner_lifecycle():
+def test_runner_lifecycle(fake_module):
     events = set()
 
     class Container(object):
-        def __init__(self, service_cls, worker_ctx_cls, config):
+        def __init__(self, service_cls, config):
             self.service_name = service_cls.__name__
             self.service_cls = service_cls
-            self.worker_ctx_cls = worker_ctx_cls
 
         def start(self):
             events.add(('start', self.service_cls.name, self.service_cls))
@@ -59,8 +60,10 @@ def test_runner_lifecycle():
         def wait(self):
             events.add(('wait', self.service_cls.name, self.service_cls))
 
-    config = {}
-    runner = ServiceRunner(config, container_cls=Container)
+    fake_module.ServiceContainer = Container
+
+    config = {'SERVICE_CONTAINER_CLS': 'fake_module.ServiceContainer'}
+    runner = ServiceRunner(config)
 
     runner.add_service(TestService1)
     runner.add_service(TestService2)
@@ -94,14 +97,13 @@ def test_runner_lifecycle():
     }
 
 
-def test_contextual_lifecycle():
+def test_contextual_lifecycle(fake_module):
     events = set()
 
     class Container(object):
-        def __init__(self, service_cls, worker_ctx_cls, config):
+        def __init__(self, service_cls, config):
             self.service_name = service_cls.__name__
             self.service_cls = service_cls
-            self.worker_ctx_cls = worker_ctx_cls
 
         def start(self):
             events.add(('start', self.service_cls.name, self.service_cls))
@@ -112,13 +114,11 @@ def test_contextual_lifecycle():
         def kill(self, exc=None):
             events.add(('kill', self.service_cls.name, self.service_cls))
 
-        def wait(self):
-            events.add(('wait', self.service_cls.name, self.service_cls))
+    fake_module.ServiceContainer = Container
 
-    config = {}
+    config = {'SERVICE_CONTAINER_CLS': 'fake_module.ServiceContainer'}
 
-    with run_services(config, TestService1, TestService2,
-                      container_cls=Container):
+    with run_services(config, TestService1, TestService2):
         # Ensure the services were started
         assert events == {
             ('start', 'foobar_1', TestService1),
@@ -134,8 +134,7 @@ def test_contextual_lifecycle():
     }
 
     events = set()
-    with run_services(config, TestService1, TestService2,
-                      container_cls=Container, kill_on_exit=True):
+    with run_services(config, TestService1, TestService2, kill_on_exit=True):
         # Ensure the services were started
         assert events == {
             ('start', 'foobar_1', TestService1),
@@ -151,9 +150,9 @@ def test_contextual_lifecycle():
     }
 
 
-def test_runner_waits_raises_error():
+def test_runner_waits_raises_error(fake_module):
     class Container(object):
-        def __init__(self, service_cls, worker_ctx_cls, config):
+        def __init__(self, service_cls, config):
             pass
 
         def start(self):
@@ -162,13 +161,14 @@ def test_runner_waits_raises_error():
         def stop(self):
             pass
 
-        def kill(self, exc):
-            pass
-
         def wait(self):
             raise Exception('error in container')
 
-    runner = ServiceRunner(config={}, container_cls=Container)
+    fake_module.ServiceContainer = Container
+
+    config = {'SERVICE_CONTAINER_CLS': 'fake_module.ServiceContainer'}
+
+    runner = ServiceRunner(config=config)
     runner.add_service(TestService1)
     runner.start()
 
@@ -177,13 +177,14 @@ def test_runner_waits_raises_error():
     assert exc_info.value.args == ('error in container',)
 
 
-def test_multiple_runners_coexist(runner_factory, rabbit_config,
-                                  rabbit_manager):
+def test_multiple_runners_coexist(
+    runner_factory, rabbit_config, rabbit_manager, service_cls, tracker
+):
 
-    runner1 = runner_factory(rabbit_config, Service)
+    runner1 = runner_factory(rabbit_config, service_cls)
     runner1.start()
 
-    runner2 = runner_factory(rabbit_config, Service)
+    runner2 = runner_factory(rabbit_config, service_cls)
     runner2.start()
 
     vhost = rabbit_config['vhost']
@@ -200,70 +201,64 @@ def test_multiple_runners_coexist(runner_factory, rabbit_config,
     assert_stops_raising(check_consumers)
 
     # test events (both services will receive if in "broadcast" mode)
-    event_data = "msg"
-    with event_dispatcher('srcservice', rabbit_config) as dispatch:
-        dispatch(TestEvent(event_data))
+    event_data = "event"
+    dispatch = event_dispatcher(rabbit_config)
 
-    with eventlet.Timeout(1):
-        while len(received) < 2:
-            eventlet.sleep()
+    container1 = list(runner1.containers)[0]
+    container2 = list(runner2.containers)[0]
 
-        assert received == [event_data, event_data]
+    with entrypoint_waiter(container1, "handle"):
+        with entrypoint_waiter(container2, "handle"):
+            dispatch('srcservice', "testevent", event_data)
+    assert tracker.call_args_list == [call(event_data), call(event_data)]
 
     # verify there are two consumers on the rpc queue
     rpc_queue = rabbit_manager.get_queue(vhost, 'rpc-service')
     assert rpc_queue['consumers'] == 2
 
     # test rpc (only one service will respond)
-    del received[:]
-    arg = "msg"
-    with RpcProxy('service', rabbit_config) as proxy:
+    arg = "arg"
+    with ServiceRpcProxy('service', rabbit_config) as proxy:
         proxy.handle(arg)
 
-    with eventlet.Timeout(1):
-        while len(received) == 0:
-            eventlet.sleep()
-
-        assert received == [arg]
+    assert tracker.call_args_list == [
+        call(event_data), call(event_data), call(arg)
+    ]
 
 
-def test_runner_with_duplicate_services(runner_factory, rabbit_config):
+def test_runner_with_duplicate_services(
+    runner_factory, rabbit_config, service_cls, tracker
+):
 
     # host Service multiple times
     runner = runner_factory(rabbit_config)
-    runner.add_service(Service)
-    runner.add_service(Service)  # no-op
+    runner.add_service(service_cls)
+    runner.add_service(service_cls)  # no-op
     runner.start()
 
     # it should only be hosted once
     assert len(runner.containers) == 1
+    container = list(runner.containers)[0]
 
     # test events (only one service is hosted)
-    event_data = "msg"
-    with event_dispatcher('srcservice', rabbit_config) as dispatch:
-        dispatch(TestEvent(event_data))
+    event_data = "event"
+    dispatch = event_dispatcher(rabbit_config)
 
-    with eventlet.Timeout(1):
-        while len(received) == 0:
-            eventlet.sleep()
-
-        assert received == [event_data]
+    with entrypoint_waiter(container, "handle"):
+        dispatch('srcservice', "testevent", event_data)
+    assert tracker.call_args_list == [call(event_data)]
 
     # test rpc
-    arg = "msg"
-    del received[:]
-
-    with RpcProxy("service", rabbit_config) as proxy:
+    arg = "arg"
+    with ServiceRpcProxy("service", rabbit_config) as proxy:
         proxy.handle(arg)
 
-    with eventlet.Timeout(1):
-        while len(received) == 0:
-            eventlet.sleep()
-
-        assert received == [arg]
+    assert tracker.call_args_list == [call(event_data), call(arg)]
 
 
-def test_runner_catches_managed_thread_errors(runner_factory, rabbit_config):
+def test_runner_catches_managed_thread_errors(
+    runner_factory, rabbit_config, service_cls
+):
 
     class Broken(Exception):
         pass
@@ -271,9 +266,9 @@ def test_runner_catches_managed_thread_errors(runner_factory, rabbit_config):
     def raises():
         raise Broken('error')
 
-    runner = runner_factory(rabbit_config, Service)
+    runner = runner_factory(rabbit_config, service_cls)
 
-    container = get_container(runner, Service)
+    container = get_container(runner, service_cls)
     container.spawn_managed_thread(raises)
 
     with pytest.raises(Broken):
